package cn.rongcloud.imuat;

import android.os.Parcel;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;

// 1. 自定义消息实现 MessageTag 注解
@MessageTag(value = "app:red_packet", flag = MessageTag.ISCOUNTED)
public class CustomRedPacketMessage extends MessageContent {
    // <editor-fold desc="* 2. 自身内部变量，可以有多个">
    private static final String TAG = "red_packet";
    private String content;
    // </editor-fold>

    // <editor-fold desc="* 3. 对外构造方法">
    private CustomRedPacketMessage() {}

    // 快速构建消息对象方法
    public static CustomRedPacketMessage obtain(String content) {
        CustomRedPacketMessage msg = new CustomRedPacketMessage();
        msg.content = content;
        return msg;
    }
    // </editor-fold>

    // <editor-fold desc="* 4. 二进制 Encode & decode 编解码方法">

    /**
     * 将本地消息对象序列化为消息数据。
     *
     * @return 消息数据。
     */
    @Override
    public byte[] encode() {
        //此处以需要携带 “用户信息” 或者 “@ 人” 信息为例
        JSONObject jsonObj = super.getBaseJsonObject();
        try {
            //  将所有自定义消息的内容，都序列化至 json 对象中
            jsonObj.put("content", this.content);
        } catch (JSONException e) {
            Log.e(TAG, "JSONException " + e.getMessage());
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            Log.e(TAG, "UnsupportedEncodingException ", e);
        }
        return null;
    }

    /** 创建 MyTextContent(byte[] data) 带有 byte[] 的构造方法用于解析消息内容. */
    public CustomRedPacketMessage(byte[] data) {
        if (data == null) {
            return;
        }
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            Log.e(TAG, "UnsupportedEncodingException ", e);
        }
        if (jsonStr == null) {
            Log.e(TAG, "jsonStr is null ");
            return;
        }

        try {
            JSONObject jsonObj = new JSONObject(jsonStr);
            //此处以需要携带 “用户信息” 或者 “@ 人” 信息为例
            super.parseBaseJsonObject(jsonObj);
            // 将所有自定义变量从收到的 json 解析并赋值
            if (jsonObj.has("content")) {
                content = jsonObj.optString("content");
            }
        } catch (JSONException e) {
            Log.e(TAG, "JSONException " + e.getMessage());
        }
    }

    // </editor-fold>

    // <editor-fold desc="* 5. Parcel 的序列化方法">

    @Override
    public void writeToParcel(Parcel dest, int i) {
        // 对消息属性进行序列化，将类的数据写入外部提供的 Parcel 中，此处以需要序列化"用户信息"，"@信息" 等为例
        super.writeToBaseInfoParcel(dest);
        ParcelUtils.writeToParcel(dest, content);
    }
    /**
     * 构造函数。
     *
     * @param in 初始化传入的 Parcel。
     */
    public CustomRedPacketMessage(Parcel in) {
//      此处以需要序列化"用户信息"，"@信息" 等为例
        super.readFromBaseInfoParcel(in);
        setContent(ParcelUtils.readFromParcel(in));
    }

    public static final Creator<CustomRedPacketMessage> CREATOR =
            new Creator<CustomRedPacketMessage>() {
                public CustomRedPacketMessage createFromParcel(Parcel source) {
                    return new CustomRedPacketMessage(source);
                }

                public CustomRedPacketMessage[] newArray(int size) {
                    return new CustomRedPacketMessage[size];
                }
            };
    @Override
    public int describeContents() {
        return 0;
    }
    // </editor-fold>

    // <editor-fold desc="* 6. get & set 方法">
    /**
     * 设置消息的内容。
     *
     * @param content 消息的内容。
     */
    public void setContent(String content) {
        this.content = content;
    }

    public String getContent() {
        return content;
    }
    // </editor-fold>

}